package LIC.UC04v1.controllers;

import javax.validation.Valid;

import LIC.UC04v1.model.User;
import LIC.UC04v1.repositories.RoleRepository;
import LIC.UC04v1.repositories.UserRepository;
import LIC.UC04v1.services.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;
/*-----------------------------------------------------------------------
* This Class handles requests in relation to the home page and modification
* of admins and their privileges.
-----------------------------------------------------------------------*/
@RequestMapping("/admin")
@Controller
public class AdminController {

    @Autowired
    private UserService userService;

    @Autowired
    private RoleRepository roleRepository;

    @Autowired
    private UserRepository userRepository;

    //URL mapping for "admin.html" page
    @RequestMapping(path = "/adminOptions")
    public String adminOps() {
        return "admin";
    }

    //creates new user model to be bound with info from "registration1.html" page
    @RequestMapping(value="/addAdmin", method = RequestMethod.GET)
    public ModelAndView registration(){
        ModelAndView modelAndView = new ModelAndView();
        User user = new User();
        modelAndView.addObject("user", user);
        modelAndView.setViewName("registration");
        return modelAndView;
    }


    //This functions handles the requests an admin makes when trying to add a new admin
    @RequestMapping(value = "/registration", method = RequestMethod.POST)
    public ModelAndView createNewUser(@Valid User user, BindingResult bindingResult) {
        ModelAndView modelAndView = new ModelAndView();
        User userExists = userRepository.findByEmail(user.getEmail());
        User userExistsUsername = userRepository.findByUsername(user.getUsername());
        if (userExists != null) { //if there are any users that already have that email
            bindingResult
                    .rejectValue("email", "error.user",
                            "There is already a user registered with the email provided");
            modelAndView.addObject("userExists", "Admin with this email already exists");
        }
        if (userExistsUsername != null) { //if any users are using the username
            modelAndView.addObject("userExistsUN", "Admin with this username already exists");
        }
        if (bindingResult.hasErrors()) { //if there are any errors
            modelAndView.setViewName("registration"); //set view for model
        } else {
            userService.saveOrUpdate(user); //save user to the database
            modelAndView.addObject("successMessage", "User has been registered successfully");
            modelAndView.addObject("user", new User());
            modelAndView.setViewName("registration");
        }
        return modelAndView;
    }
    /*-----------------------------------------------------------------------
    * This method simply forwards the user to the delete user page.
    -----------------------------------------------------------------------*/
    @RequestMapping(path = "/removeAdmin")
    public String brute() {
        return "deleteUser";
    }
    /*-----------------------------------------------------------------------
    * This Method accepts a user as a parameter and deletes said user if they
    * exist in the system.
    -----------------------------------------------------------------------*/
    @RequestMapping(value = "/deleteUser", method = RequestMethod.POST)
    public ModelAndView deleteUser(@RequestParam("username") String username){
        ModelAndView modelAndView = new ModelAndView();
        //find admin in the system
        User user = userRepository.findByUsername(username);
        if(user != null){
            roleRepository.findById(2).get().getUsers().remove(user);
            userRepository.delete(user);
            modelAndView.addObject("successDelete", "User has been successfully deleted");
            modelAndView.setViewName("deleteUser");
        }else{
            modelAndView.addObject("errorMessage", "No user with this username exists");
            modelAndView.setViewName("deleteUser");
        }
        return modelAndView;
    }
    /*-----------------------------------------------------------------------
    * This Method sets the view to the home page
    * Anyone who is not an admin will be denied access
    -----------------------------------------------------------------------*/
    @RequestMapping(value="/home", method = RequestMethod.GET)
    public ModelAndView home(){
        ModelAndView modelAndView = new ModelAndView();
        Authentication auth = SecurityContextHolder.getContext().getAuthentication();
        User user = userService.findByUserName(auth.getName());
        //modelAndView.addObject("userName", "Welcome " + user.getName() + " " + user.getLastName() + " (" + user.getEmail() + ")");
        modelAndView.addObject("adminMessage","Content Available Only for Users with Admin Role");
        modelAndView.setViewName("home");
        return modelAndView;
    }

}
