package LIC.UC04v1.controllers;

import LIC.UC04v1.model.Doctor;
import LIC.UC04v1.model.Student;
import LIC.UC04v1.repositories.DoctorRepository;
import LIC.UC04v1.repositories.StudentRepository;
import org.springframework.mail.MailException;
import org.springframework.mail.javamail.JavaMailSenderImpl;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.mail.javamail.MimeMessagePreparator;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.mail.MailException;
import org.springframework.mail.javamail.JavaMailSender;
import javax.mail.*;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.Message;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.InternetHeaders;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import LIC.UC04v1.controllers.emailsLeft;

import java.io.*;
import java.util.Properties;

/*
    SOME NOTES ON THIS CLASS
        This class sends emails to all the doctors and students in the database. Doctors receive a
        unique link to the availabilities form, and students receive a unique link to build their
        schedule. Emails are currently sent from our no.reply.lic.tcu@gmail.com address. We ere sending emails
        using SMTP and gmail.
 */

@RequestMapping("/admin")
@Controller
public class EmailsController {

    private DoctorRepository doctorRepository;
    private StudentRepository studentRepository;
    private JavaMailSender mailSender;

    public EmailsController(DoctorRepository doctorRepository, StudentRepository studentRepository){
        this.doctorRepository = doctorRepository;
        this.studentRepository = studentRepository;
    }

    /*-----------------------------------------------------------------------
    * This Method simply redirects the admin to the send email page
    -----------------------------------------------------------------------*/
    @GetMapping(path = "/send-emails")
    public String getImports(Model model){
        emailsLeft e = new emailsLeft();
        int countAll = 0;
        int countLeft = 0;
        //for each doctor
        for (Student stu : studentRepository.findAll()) {
            //if the doctor matches the requested information
            countAll++;
            if (!stu.getBeenEmailed()) {
                //add availabilities to counts array
                countLeft++;
            }
        }
        e.setEmailsLeftStu(countLeft+"");
        e.setTotalEmailsStu(countAll+"");
        countAll = 0;
        countLeft = 0;
        //for each doctor
        for (Doctor doc : doctorRepository.findAll()) {
            //if the doctor matches the requested information
            countAll++;
            if (!doc.getBeenEmailed()) {
                //add availabilities to counts array
                countLeft++;
            }
        }
        e.setEmailsLeftDoc(countLeft+"");
        e.setTotalEmailsDoc(countAll+"");
        model.addAttribute("emailStats", e);

        return "sendEmails1";
    }

    /*-----------------------------------------------------------------------
   * This Method runs when an admin chooses to send either student scheduling or doctor
   * availabillity emails 100 at a time. It utilizes javax.mail 1.6.2 API and SMTP.
   -----------------------------------------------------------------------*/
    @RequestMapping(path = "/send-piecemeal-emails/{type}")
    public String piecemealEmails(Model model, @PathVariable String type) throws Exception{
        //setting up properties and credintals to send email with smtp
        JavaMailSenderImpl sender = new JavaMailSenderImpl();
        sender.setHost("smtp.gmail.com");
        sender.setUsername("no.reply.lic.tcu");
        sender.setPassword("clifbar123");
        //587 is the smtp port
        sender.setPort(587);
        Properties props = new Properties();
        props.put("mail.smtp.auth", "true");
        props.put("mail.smtp.starttls.enable", "true");
        sender.setJavaMailProperties(props);
        //initallizing email counter
        int i = 0;
        //Loop through all the doctors and send an email
        if(type.equals("doctors")) {
            for (Doctor doc : doctorRepository.findAll()) {
                //this if statment prevents sending or counting to already emailed users
                if(!doc.getBeenEmailed()) {
                    //next two lines indicates the doc has been emailed and saves to database
                    doc.setBeenEmailed(true);
                    doctorRepository.save(doc);
                    i++;
                    System.out.println("One doctor...");
                    //Message to doctor is created
                    MimeMessage message = sender.createMimeMessage();
                    MimeMessageHelper helper = new MimeMessageHelper(message);
                    InternetHeaders headers = new InternetHeaders();
                    headers.addHeader("Content-type", "text/html; charset=UTF-8");
                    String text = "localhost:8080/doctor/" + doc.getId();
                    String html = "<a href=\'"+text+"\'>Click Link</a>";
                    //MimeBodyPart body = new MimeBodyPart(headers, html.getBytes("UTF-8"));


                    helper.setTo("no.reply.lic.tcu@gmail.com");
                    helper.setText(html,true);
                    helper.setText(text);

//                    helper.setText(
//                            "<html><body>"  + doc.getName() + "<br>Thank you for offering to teach a medical school student. The next step is to indicate \n" +
//                                    "your availabilities. Please follow the unique link below and complete the form. Do not \n" +
//                                    "share the link with others:<br>" + html +
//                                    "Thank you! TCU/UNTHSC Medical School<body><html>"+
//                                    text
//                    , true);
                    //helper.setText(html, "UTF-8", "html");
                    helper.setSubject("Information Required");

                    try {
                        System.out.println("Sending Email...");
                        sender.send(message);
                        System.out.println("Doc Email Sent!");
                    } catch (Exception e) {
                        System.out.println("Error: " + e.getMessage());
                    }
                }
                //determine how many emails to send
                //if(i==10)break;
            }
            return "SentAllDocs";
        }
        else{
            //Loop through all the students and send an email
            for (Student stu: studentRepository.findAll()) {
                //this if statment prevents sending or counting to already emailed users
                if(!stu.getBeenEmailed()) {
                    i++;
                    //next two lines indicates the doc has been emailed and saves to database
                    stu.setBeenEmailed(true);
                    studentRepository.save(stu);
                    //Message to doctor is created
                    MimeMessage message = sender.createMimeMessage();
                    MimeMessageHelper helper = new MimeMessageHelper(message);

                    helper.setTo("no.reply.lic.tcu@gmail.com");
                    helper.setText(stu.getName() + ", \n " +
                            "It's time to request your clerkships. Please follow the unique link below to complete " +
                            "the process. Please do not share the link with others, as it is linked to your name. \n" +
                            "localhost:8080/" + stu.getId() +
                            "\n\nThank you!\nTCU/UNTHSC Medical School"
                    );
                    helper.setSubject("Request Clerkship Schedule");

                    try {
                        System.out.println("Sending Email...");
                        sender.send(message);
                        System.out.println("Stu Email Sent!");
                    } catch (Exception e) {
                        System.out.println("Error: " + e.getMessage());
                    }
                }
                //if(i==10)break;
            }
            return "SentAllStus";
        }
        //tell admin that emails were were sent

    }

    /*-----------------------------------------------------------------------
    * THIS METHOD IS NOT BEING USED CURRENTLY DUE TO EMAIL SEND LIMITATIONS FROM GMAIL
    * INSTEAD THE PIECEMEAL METHOD ABOVE THIS IS BEING USED
    * This Method runs when an admin chooses to send either student scheduling or doctor
    * availabillity emails. It utilizes javax.mail 1.6.2 API and SMTP.
    -----------------------------------------------------------------------*/
    @RequestMapping(path = "/send-emails/{type}")
    public String docEmails(Model model, @PathVariable String type) throws Exception{
        //setting up properties and credintals to send email with
//        JavaMailSenderImpl sender = new JavaMailSenderImpl();
//        sender.setHost("smtp.gmail.com");
//        sender.setUsername("no.reply.lic.tcu");
//        sender.setPassword("clifbar123");
        //587 is used for smtp specifically
//        sender.setPort(587);
//        Properties props = new Properties();
//        props.put("mail.smtp.auth", "true");
//        props.put("mail.smtp.starttls.enable", "true");
//        sender.setJavaMailProperties(props);

        Properties props = new Properties();
        props.put("mail.transport.protocol", "smtp");
        props.put("mail.smtp.port", 587);
        props.put("mail.smtp.starttls.enable", "true");
        props.put("mail.smtp.auth", "true");
        Session session = Session.getDefaultInstance(props);

        //Loop through all the doctors and send an email
        if(type.equals("doctors")) {
            for (Doctor doc : doctorRepository.findAll()) {
                System.out.println("One doctor...");
                MimeMessage msg = new MimeMessage(session);
//                MimeMessage message = sender.createMimeMessage();
//                MimeMessageHelper helper = new MimeMessageHelper(message);

//                //We will un-comment this. I don't want to accidentally send emails to all the doctors!!
//                //helper.setTo(doc.getEmail());
//                //For now, send emails to a junk account
//                helper.setTo("no.reply.lic.tcu@gmail.com");
//
//                helper.setText(doc.getName() + ", \n " +
                msg.setFrom(new InternetAddress("no.reply.lic.tcu@gmail.com", "Kayla Beeler"));
                msg.setRecipient(Message.RecipientType.TO, new InternetAddress("no.reply.lic.tcu@gmail.com"));
                msg.setSubject("TCU/UNTHSC Med School Availabillity Questionair");
                msg.setContent(doc.getName() + ", \n " +
                        "Thank you for offering to teach a medical school student. The next step is to indicate " +
                        "your availabilities. Please follow the unique link below and complete the form. Do not " +
                        "share the link with others. \n" +
                        "localhost:8080/doctor/" + doc.getId() +
                        "\n\nThank you!\nTCU/UNTHSC Medical School","text/html");

                //msg.setHeader("X-SES-CONFIGURATION-SET", "ConfigSet");

                Transport transport = session.getTransport();

                try
                {
                    System.out.println("Sending...");

                    // Connect to Amazon SES using the SMTP username and password you specified above.
                    transport.connect("email-smtp.us-west-2.amazonaws.com", "AKIA5DFOVNYYK4NWH2ID", "BHV4gd5so0KMtbMjJxzarRySeHZHby5IaGpYY6nQgM53");

                    // Send the email.
                    transport.sendMessage(msg, msg.getAllRecipients());
                    System.out.println("Email sent!");
//                        "\n\nThank you!\nTCU/UNTHSC Medical School"
//                );
//                helper.setSubject("Information Required");
//
//                try {
//                    System.out.println("Sending Email...");
//                    sender.send(message);
//                    System.out.println("Doc Email Sent!");
                }
                catch (Exception ex) {
                    System.out.println("The email was not sent.");
                    System.out.println("Error message: " + ex.getMessage());
                }
                finally
                {
                    // Close and terminate the connection.
                    transport.close();
                }
                return "SentAllDocs";
//                catch(Exception e){
//                    System.out.println("Error: " + e.getMessage());
//                }
            }
        }
        else{
            for (Student stu: studentRepository.findAll()) {
//                MimeMessage message = sender.createMimeMessage();
//                MimeMessageHelper helper = new MimeMessageHelper(message);
                MimeMessage msg = new MimeMessage(session);
                //We will un-comment this. I don't want to accidentally send emails to all the students!!
                //helper.setTo(doc.getEmail());
                //For now, send emails to a junk account
//                helper.setTo("no.reply.lic.tcu@gmail.com");
//                helper.setText(stu.getName() + ", \n " +
                msg.setFrom(new InternetAddress("no.reply.lic.tcu@gmail.com", "Kayla Beeler"));
                msg.setRecipient(Message.RecipientType.TO, new InternetAddress("no.reply.lic.tcu@gmail.com"));
                msg.setSubject("TCU/UNTHSC Student Clerkship Schedule");
                msg.setContent(stu.getName() + ", \n " +
                        "It's time to request your clerkships. Please follow the unique link below to complete " +
                        "the process. Please do not share the link with others, as it is linked to your name. \n" +
                        "localhost:8080/student/" + stu.getId() +
                        "\n\nThank you!\nTCU/UNTHSC Medical School", "text/html");

                //msg.setHeader("X-SES-CONFIGURATION-SET", "ConfigSet");

                Transport transport = session.getTransport();

                try {
                    System.out.println("Sending...");

                    // Connect to Amazon SES using the SMTP username and password you specified above.
                    transport.connect("email-smtp.us-west-2.amazonaws.com", "AKIA5DFOVNYYK4NWH2ID", "BHV4gd5so0KMtbMjJxzarRySeHZHby5IaGpYY6nQgM53");

                    // Send the email.
                    transport.sendMessage(msg, msg.getAllRecipients());
                    System.out.println("Email sent!");
                } catch (Exception ex) {
                    System.out.println("The email was not sent.");
                    System.out.println("Error message: " + ex.getMessage());

//                        "localhost:8080/" + stu.getId() +
//                        "\n\nThank you!\nTCU/UNTHSC Medical School"
//                );
//                helper.setSubject("Request Clerkship Schedule");
//
//                try {
//                    System.out.println("Sending Email...");
//                    sender.send(message);
//                    System.out.println("Stu Email Sent!");
                }
//                catch(Exception e){
//                    System.out.println("Error: " + e.getMessage());
//                }
                finally {
                    // Close and terminate the connection.
                    transport.close();
                }
                return "SentAllStus";
            }
        }
        return "emailsSent";
    }
}
