package LIC.UC04v1.services;

import LIC.UC04v1.controllers.doctorForm;
import LIC.UC04v1.repositories.DoctorRepository;
import LIC.UC04v1.model.Doctor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashSet;
import java.util.Optional;
import java.util.Set;
/*-----------------------------------------------------------------------
* This class handles methods that pass information in between the controller and the database.
* Specifically tailored to doctors.
-----------------------------------------------------------------------*/
@Slf4j
@Service
public class DoctorServiceImpl implements DoctorService{
    private final DoctorRepository doctorRepository;

    public DoctorServiceImpl(DoctorRepository doctorRepository) {
        this.doctorRepository = doctorRepository;
    }



    /*-----------------------------------------------------------------------
    * This Method finds a specific doctor in the database
    -----------------------------------------------------------------------*/
    @Override
    public Doctor findById(String l){
        Optional<Doctor> doctorOptional = doctorRepository.findById(l);

        if(!doctorOptional.isPresent()){
            throw new RuntimeException("Doctor Not Found!");
        }


        return doctorOptional.get();
    }


    /*-----------------------------------------------------------------------
    * This Method updates a doctor in the database from the form information.
    -----------------------------------------------------------------------*/
    @Override
    @Transactional
    public doctorForm saveDoctorForm(doctorForm docForm) {
        //find doctor
        Doctor detachedDoctor = findById(docForm.id);
        //update information
        String availStr = "";
        if(docForm.MM) availStr += "1";
        else availStr += "0";
        if(docForm.MA) availStr += "1";
        else availStr += "0";
        if(docForm.TM) availStr += "1";
        else availStr += "0";
        if(docForm.TA) availStr += "1";
        else availStr += "0";
        if(docForm.WM) availStr += "1";
        else availStr += "0";
        if(docForm.WA) availStr += "1";
        else availStr += "0";
        if(docForm.RM) availStr += "1";
        else availStr += "0";
        if(docForm.RA) availStr += "1";
        else availStr += "0";
        if(docForm.FM) availStr += "1";
        else availStr += "0";
        if(docForm.FA) availStr += "1";
        else availStr += "0";
        if(docForm.AM) availStr += "1";
        else availStr += "0";
        if(docForm.AA) availStr += "1";
        else availStr += "0";
        String a = docForm.address;
        detachedDoctor.setNumStu(Integer.parseInt(docForm.stuCount));
        availStr = availStr + availStr;
        availStr += availStr;
        detachedDoctor.setAddress(a);
        detachedDoctor.setLocation(docForm.location);
        detachedDoctor.setAvailabilities(availStr);
        //update doc in db
        doctorRepository.save(detachedDoctor);
        return null;
    }

    @Override
    public Set<Doctor> getDoctors(){

        Set<Doctor> doctorSet = new HashSet<>();
        doctorRepository.findAll().iterator().forEachRemaining(doctorSet::add);

        return doctorSet;
    }
}
